@@ -7,6 +7,7 @@ from openpyxl.utils import get_column_letter
 from dataclasses import dataclass, asdict
 from tqdm import trange
 import sys
+import matplotlib.pyplot as plt
 
 from openpyxl.cell.cell import Cell
 from openpyxl.styles import Font
@@ -348,7 +349,7 @@ class ParametersLEDET:
         except:
             setattr(getattr(self, LEDETclass), attribute, imitate)
 
-    def readLEDETExcel(self, file):
+    def readLEDETExcel(self, file, verbose: bool = True):
         ##File must be whole eos string
         workbookVariables = openpyxl.load_workbook(file, data_only=True)
 
@@ -379,7 +380,7 @@ class ParametersLEDET:
             except TypeError as e:
                 if attribute in self.Inputs.__annotations__: raise e
                 if attribute=='None' or attribute==None: continue
-                print("Error with attribute: {}, continuing.".format(attribute))
+                if verbose: print("Error with attribute: {}, continuing.".format(attribute))
         #Options
         worksheetOptions = workbookVariables['Options']
         for i in range(1, worksheetOptions.max_row+1):
@@ -396,7 +397,7 @@ class ParametersLEDET:
             except  TypeError as e:
                 if attribute in self.Options.__annotations__: raise e
                 if attribute == 'None' or attribute == None: continue
-                print("Error with attribute: {}, continuing.".format(attribute))
+                if verbose: print("Error with attribute: {}, continuing.".format(attribute))
         #Plots
         worksheetPlots = workbookVariables['Plots']
         for i in range(1, worksheetPlots.max_row+1):
@@ -415,7 +416,7 @@ class ParametersLEDET:
                     self.setAttribute(self.Plots, attribute, value)
             except  TypeError as e:
                 if attribute == 'None' or attribute == None: continue
-                print("Error with attribute: {}, continuing.".format(attribute))
+                if verbose: print("Error with attribute: {}, continuing.".format(attribute))
         # Variables
         try:
             worksheetVariables = workbookVariables['Variables']
@@ -434,7 +435,7 @@ class ParametersLEDET:
                 except TypeError as e:
                     if attribute in self.Variables.__annotations__: raise e
                     if attribute == 'None' or attribute == None: continue
-                    print("Error with attribute: {}, continuing.".format(attribute))
+                    if verbose: print("Error with attribute: {}, continuing.".format(attribute))
         except:
             pass
             print("Error while reading Variables. Please check!")
@@ -1009,6 +1010,102 @@ class ParametersLEDET:
         else:
             return
 
+    def setHeliumFraction(self, PercentVoids):
+        if np.max(self.Inputs.nStrands_inGroup)==1:
+            print('You are about to set a helium-fraction for a single-stranded wire!')
+
+        if not isinstance(self.Inputs.wBare_inGroup, np.ndarray):
+            self.Inputs.wBare_inGroup = np.array(self.Inputs.wBare_inGroup)
+        if not isinstance(self.Inputs.hBare_inGroup, np.ndarray):
+            self.Inputs.hBare_inGroup = np.array(self.Inputs.hBare_inGroup)
+        if not isinstance(self.Inputs.wIns_inGroup, np.ndarray):
+            self.Inputs.wIns_inGroup = np.array(self.Inputs.wIns_inGroup)
+        if not isinstance(self.Inputs.hIns_inGroup, np.ndarray):
+            self.Inputs.hIns_inGroup = np.array(self.Inputs.hIns_inGroup)
+        if not isinstance(self.Inputs.ds_inGroup, np.ndarray):
+            self.Inputs.ds_inGroup = np.array(self.Inputs.ds_inGroup)
+        if not isinstance(self.Inputs.nStrands_inGroup, np.ndarray):
+            self.Inputs.nStrands_inGroup = np.array(self.Inputs.nStrands_inGroup)
+
+        cs_bare = self.Inputs.wBare_inGroup*self.Inputs.hBare_inGroup
+        cs_ins = (self.Inputs.wBare_inGroup +2*self.Inputs.wIns_inGroup)* \
+                (self.Inputs.hBare_inGroup +2*self.Inputs.hIns_inGroup)
+        cs_strand = self.Inputs.nStrands_inGroup*np.pi*(self.Inputs.ds_inGroup**2)/4
+        strand_total = cs_strand/cs_ins
+        ins_total = (cs_ins - cs_bare)/cs_ins
+        VoidRatio = (cs_bare - cs_strand)/cs_ins
+        extVoids = VoidRatio - (PercentVoids/100.0)
+        if any(sV < 0 for sV in extVoids):
+            print("Negative externalVoids calculated. Abort, please check.")
+            return
+        nGroups = len(self.Inputs.nT)
+        self.Inputs.overwrite_f_externalVoids_inGroup = extVoids
+        self.Inputs.overwrite_f_internalVoids_inGroup = np.ones((nGroups,)).transpose()*(PercentVoids/100.0)
+
+    def preparePersistentCurrents(self, I_PC_LUT, dIdt, timeStep):
+        # LUT controlling power supply, Current [A]. Two cycles of ramping from 0 to nominal current and back to zero
+        if isinstance(I_PC_LUT,list):
+            I_PC_LUT = np.array(I_PC_LUT)
+        self.Inputs.I_PC_LUT = I_PC_LUT
+        self.Inputs.I00 = 0
+
+        # LUT controlling power supply, Time [s]
+        t_PC_LUT = np.zeros(len(self.Inputs.I_PC_LUT))
+        # Generates a time LUT that is dependent on the ramp rate of the current.
+        for x in range(len(self.Inputs.I_PC_LUT)):
+            if x == 0:  t_PC_LUT[x] = 0
+            elif x == 1: t_PC_LUT[x] = 0.1
+            elif x % 2 == 1: t_PC_LUT[x] = t_PC_LUT[x - 1] + 1
+            elif x % 4 == 0:
+                t_PC_LUT[x] = t_PC_LUT[x - 1] - (self.Inputs.I_PC_LUT[x] - self.Inputs.I_PC_LUT[x - 1]) / dIdt
+            elif (x + 2) % 4 == 0:
+                t_PC_LUT[x] = t_PC_LUT[x - 1] + (self.Inputs.I_PC_LUT[x] - self.Inputs.I_PC_LUT[x - 1]) / dIdt
+            else: continue
+        self.Inputs.t_PC_LUT =  t_PC_LUT
+
+        # time vector - Generates a time vector with finer timestepping when the ramp rate of the current changes
+        nElements = (len(self.Inputs.I_PC_LUT)-2)*6+3
+        time_vector_params = np.zeros(nElements)
+        every_sixth_element = range(nElements-3)[::6]
+        for x in every_sixth_element:
+            time_vector_params[x] = time_vector_params[x - 1] + timeStep
+            time_vector_params[x + 1] = timeStep
+            time_vector_params[x + 2] = t_PC_LUT[(x // 6) + 1] - 0.02
+            time_vector_params[x + 3] = time_vector_params[x + 2] + 0.001
+            time_vector_params[x + 4] = 0.001
+            time_vector_params[x + 5] = time_vector_params[x + 2] + 0.04
+        time_vector_params[0] = 0
+        time_vector_params[1] = 0.010
+        time_vector_params[-1] = t_PC_LUT[-1]
+        time_vector_params[-2] = 0.5
+        time_vector_params[-3] = time_vector_params[-4]+0.5
+        self.Options.time_vector_params = time_vector_params
+
+        # Changes in options
+        if np.all(self.Inputs.f_SC_strand_inGroup == self.Inputs.f_SC_strand_inGroup[0]):
+            self.Options.flag_hotSpotTemperatureInEachGroup = 0
+        else:
+            self.Options.flag_hotSpotTemperatureInEachGroup = 0
+        self.Options.minCurrentDiode = 0
+        self.Options.flag_persistentCurrents = 1
+
+        # Changes in input
+        self.Inputs.t_PC = 99999
+        self.Inputs.tQH = np.array([99999]*len(self.Inputs.tQH))
+        self.Inputs.tEE = 99999
+        self.Inputs.tQuench = np.array([t_PC_LUT[-2]]*len(self.Inputs.M_m))
+        self.Inputs.tStartQuench = np.array([99999]*len(self.Inputs.tStartQuench))
+
+        selectedFont = {'fontname': 'DejaVu Sans', 'size': 14}
+        plt.figure(figsize=(5, 5))
+        plt.plot(self.Inputs.t_PC_LUT, self.Inputs.I_PC_LUT, 'ro-', label='LUT')
+        plt.xlabel('Time [s]', **selectedFont)
+        plt.ylabel('Current [A]', **selectedFont)
+        plt.title('Look-up table controlling power supply', **selectedFont)
+        plt.grid(True)
+        plt.rcParams.update({'font.size': 12})
+        plt.show()
+
     def addVariablesInputs(self,
                            T00, l_magnet, I00, M_m,
                            fL_I, fL_L,
@@ -1514,7 +1611,7 @@ class ParametersLEDET:
             Break = 1
         return Break
 
-    def writeFileLEDET(self, nameFileLEDET, verbose: bool = False, SkipConsistencyCheck: bool = False):
+    def writeFileLEDET(self, nameFileLEDET, verbose: bool = True, SkipConsistencyCheck: bool = False):
         '''
             **Writes LEDET input file **
 
@@ -1534,7 +1631,7 @@ class ParametersLEDET:
                 print("Variables are not consistent! Writing aborted - ", nameFileLEDET)
                 return
             else:
-                print("Preliminary consistency check was successful! - ", nameFileLEDET)
+                if verbose: print("Preliminary consistency check was successful! - ", nameFileLEDET)
         else:
             print("Skipped consistency checks.")
 
@@ -1567,9 +1664,10 @@ class ParametersLEDET:
 
         # Display time stamp and end run
         currentDT = datetime.datetime.now()
-        print(' ')
-        print('Time stamp: ' + str(currentDT))
-        print('New file ' + nameFileLEDET + ' generated.')
+        if verbose:
+            print(' ')
+            print('Time stamp: ' + str(currentDT))
+            print('New file ' + nameFileLEDET + ' generated.')
         return
 
     def writeLEDETInputsNew(self, book, sheet, variableGroup, variableLabels):
