@@ -5,7 +5,7 @@ from dataclasses import dataclass, asdict
 from steam_nb_api.ledet.ParametersLEDET import ParametersLEDET
 import steam_nb_api.ledet.ParameterSweep
 
-def RunSimulations(LEDETFolder, LEDETExe, MagnetName, Simulations = 'All', RunSimulations = True):
+def RunSimulations(LEDETFolder, LEDETExe, MagnetName, Simulations = 'All', RunSimulations = False):
     ExcelFolder = LEDETFolder + "//LEDET//" + MagnetName + "//Input//"
     StartFile = LEDETFolder + "//startLEDET.xlsx"
     SimNumbers = []
@@ -23,7 +23,7 @@ def RunSimulations(LEDETFolder, LEDETExe, MagnetName, Simulations = 'All', RunSi
     else:
         SimNumbers = Simulations
 
-    df = pd.read_excel(StartFile, header=None, engine='openpyxl')
+    df = pd.read_excel(StartFile, header=None)
     df.rename(columns={0: 'a', 1: 'b', 2: 'c'}, inplace=True)
     df.loc[df['b'] == 'currFolder', 'c'] = LEDETFolder + "\\LEDET"
     df.loc[df['b'] == 'nameMagnet', 'c'] = MagnetName
