@@ -36,6 +36,9 @@ class ParametersCOSIM:
         else: self.nameMagnet = nameMagnet
         self.executionOrder = [0]
         self.SimulationNumber = [0]
+        self.t0 = [0]
+        self.tEnd = [0]
+        self.tStepMax = [0]
 
         # Load and set the default config files (using ResourceReader allows reading from a "hidden" resource folder)
         self.nameTemplateConfigFileCosim = ResourceReader.getResourcePath(os.path.join('sing', 'STEAMConfig.json'))
@@ -183,7 +186,7 @@ class ParametersCOSIM:
                 makeCopyFile(fileName_complementaryIOPortDefinition,
                              os.path.join(nameFolderCosimModel, 'LEDET'+str(i), 'LedetInputOutputPortDefinition.json'))
 
-    def makeGenericIOPortFiles(self, CoilSections, ModelFolder, CoSimFolder: str, PSpiceExecutable: str, LEDETExecutable: str,
+    def makeGenericIOPortFiles(self, CoilSections, ModelFolder, CoSimFolder: str, PSpiceExecutable: str, LEDETExecutable: str, verbose: bool = False,
                                t_0 = [0, 2e-5], t_end = [2e-5, 0.5], t_step_max = [[1e-5, 1e-5], [1e-5, 1e-5]],
                                relTolerance = [1e-4, None], absTolerance = [1, None], executionOrder = [1, 2],
                                executeCleanRun = [True, True], N_LEDET = 1, QuenchMagnet = 0, SimulationNumber = [0], DistinctMagnets = 1, PSPICEinitialConditions = []):
@@ -192,6 +195,10 @@ class ParametersCOSIM:
         elif not ModelFolder.endswith('\\'): ModelFolder = ModelFolder + '\\'
         if len(SimulationNumber) < N_LEDET: self.SimulationNumber = SimulationNumber + (N_LEDET - len(SimulationNumber))*[0]
         else: self.SimulationNumber = SimulationNumber
+
+        self.t0 = t_0
+        self.tEnd = t_end
+        self.tStepMax = t_step_max
         # Components which are used in the Ports
         Components = ["L", "CoilSections"]  # [0] = PSpice, [1]=LEDET
         # Setting the Coilsections
@@ -289,14 +296,16 @@ class ParametersCOSIM:
         else:
             if len(SimulationNumber) != N_LEDET:
                 SimulationNumber = [0]*N_LEDET
-                print("Simulation Number 0 is used for all LEDET models")
+                if verbose: print("Simulation Number 0 is used for all LEDET models")
             for i in range(1,N_LEDET+1):
                 filename = os.path.join(self.nameFolderCosimModel,'LEDET_'+str(i),'LedetConfig.json')
                 with open(filename, 'r') as f:
                     data = json.load(f)
                     if executionOrder.count(2) > 1:
                         idxLEDET = LEDETExecutable.find('LEDET')
-                        data["solverPath"] = LEDETExecutable[0:idxLEDET + 5] + "_"+str(i) + LEDETExecutable[idxLEDET + 5:]
+                        idxLEDET2 = LEDETExecutable.find('LEDET_v')
+                        idxLEDET3 = LEDETExecutable[idxLEDET:].find('\\')
+                        data["solverPath"] = LEDETExecutable[0:idxLEDET + idxLEDET3]+"\\LEDET" + "_"+str(i) + LEDETExecutable[idxLEDET2-1:]
                         print('COSIM Parallel mode: LEDET executable ',i,' should be: ', data["solverPath"],'.')
                     else: data["solverPath"] = LEDETExecutable
                     data["modelName"] = self.nameMagnet[i-1]
@@ -479,6 +488,11 @@ class ParametersCOSIM:
     def CopyCircuit(self, FileOld, FileNew):
         inOption = 0
         inAutoConverge = 0
+        delete = 0
+        if FileOld==FileNew:
+            os.rename(FileOld,FileOld+'Old')
+            FileOld = FileOld+'Old'
+            delete = 1
 
         fN = open(FileNew, 'w+')
         with open(FileOld) as fp:
@@ -497,6 +511,8 @@ class ParametersCOSIM:
                     inAutoconverge = 0
                     fN.write(ls)
             fN.close()
+        if delete:
+            os.remove(FileOld)
 
     def copyCOSIMfiles(self, nameFileSING, StimulusFile, nameMagnet, N_LEDET=1, LEDET_only = 0, ManuallyStimulusFile = ''):
         LEDETFiles = os.path.abspath(os.path.join(os.path.abspath(os.path.join(os.getcwd(), os.pardir)),os.pardir))
@@ -517,8 +533,8 @@ class ParametersCOSIM:
                 os.mkdir(nameFolderPSpiceModel)
 
             self.CopyCircuit(nameFileSING, os.path.join(nameFolderPSpiceModel, 'Circuit.cir'))
-            # makeCopyFile(nameFileSING, os.path.join(nameFolderPSpiceModel, 'Circuit.cir'))
-            makeCopyFile(StimulusFile, os.path.join(nameFolderPSpiceModel, 'ExternalStimulus.stl'))
+            if not StimulusFile.endswith( 'ExternalStimulus.stl'):
+                makeCopyFile(StimulusFile, os.path.join(nameFolderPSpiceModel, 'ExternalStimulus.stl'))
 
         LEDET_Fs = []
         for i in range(N_LEDET):
@@ -575,6 +591,9 @@ class ParametersCOSIM:
                 try:
                     if self.executionOrder[i+1] == num:
                         TurnNumber = TurnNumber + Tnew
+                        Snew = 0
+                        for k in range(len(self.t0)):
+                            Snew = Snew + (self.tEnd[k]-self.t0[k])/self.tStepMax[k][0]
                         steps = steps + Snew
                 except:
                     TurnNumber = Tnew
@@ -592,6 +611,7 @@ class ParametersCOSIM:
             print("You are trying to simulate ",str(TurnNumber_All),"in parallel mode. \n")
             maxN_PAR = np.floor(10000/TurnNumber)
             print("Consider not to user more than ", str(maxN_PAR)," parallel Simulations.")
+        return files
 
     def prepareSingleLEDETFile(self, file):
         a = ParametersLEDET()
@@ -600,7 +620,7 @@ class ParametersCOSIM:
         a.Options.flag_generateReport = 0
         a.Options.flag_saveMatFile = 0
         a.Options.flag_automaticRefinedTimeStepping = 0
-        a.writeFileLEDET(file)
+        a.writeFileLEDET(file, verbose = False)
 
         TurnNumber = max(a.Inputs.HalfTurnToInductanceBlock)
         tV = a.Options.time_vector_params
@@ -613,7 +633,7 @@ class ParametersCOSIM:
     def __findOccurrences(self, s, ch):
         return [i for i, letter in enumerate(s) if letter == ch]
 
-    def writeCOSIMBatch(self, COSIMfolder, COSIMexe, Destination = '', LEDET_exe = ''):
+    def writeCOSIMBatch(self, COSIMfolder, COSIMexe, Destination = '', LEDET_exe = '', Append: bool = False):
         if type(COSIMfolder) != list:
             COSIMfolder = [COSIMfolder]
         stub = ''
@@ -621,10 +641,10 @@ class ParametersCOSIM:
             LEDET_exe = LEDET_exe.replace('\ ', '\\')
             idx = self.__findOccurrences(LEDET_exe, '\\')[-1]
             stub = LEDET_exe[:idx]+'\\'
-        if Destination: fileName = Destination+'\\COSIM_nb_Batch.bat'
+        if Destination: fileName = os.path.join(Destination,'COSIM_nb_Batch.bat')
         else: fileName = 'COSIM_nb_Batch.bat'
-        fN = open(fileName, 'w+')
-
+        if Append: fN = open(fileName, 'a+')
+        else: fN = open(fileName, 'w+')
         txt_stub = "java -jar " + COSIMexe
         for i in range(len(COSIMfolder)):
             if '_L_' in str(COSIMfolder[i]):
@@ -644,4 +664,11 @@ class ParametersCOSIM:
                 tx_line = txt_stub + " " + str(COSIMfolder[i]) + "\\STEAMConfig.json" + " \n"
                 fN.write(tx_line)
         fN.close()
+        currentDT = datetime.datetime.now()
+        print(' ')
+        if not Append:
+            print('Simulation-Batch containing ',len(COSIMfolder),' simulations created.')
+        else:
+            print(len(COSIMfolder), ' simulations to batch-file appended.')
+        print('Time stamp: ' + str(currentDT))
 
