@@ -326,7 +326,7 @@ class CircuitFrequencyModelling:
         netlist.add(CommentElement(" Voltage measurement"));
         name = "E_total_voltage"
         nodes[0], nodes[1] = GROUND_NODE, "0total_voltage"
-        value = "VALUE = {V(" + str.format("{}out" , k-1) + ")-V(0series_out)}"
+        value = "VALUE = {V(0series_out)-V(" + str.format("{}out" , k-1) + ")}"
         netlist.add(GeneralElement(name, nodes, value));
 
         netlist.setSolver(ACSolverElement.Builder()
@@ -1554,7 +1554,7 @@ class FrequencyModelling:
         # Voltage source
         netlist.add(CommentElement(" Voltage source"));
         name = "V_AC"
-        nodes[0], nodes[1] = nodesL[0][0], nodesR[len(L) - 1][1]
+        nodes[0], nodes[1] = nodesR[len(L) - 1][1], nodesL[0][0]
         value = "AC 1"
         netlist.add(GeneralElement(name, nodes, value));
 
@@ -1569,7 +1569,7 @@ class FrequencyModelling:
         netlist.add(CommentElement(" Voltage measurement"));
         name = "E_total_voltage"
         nodes[0], nodes[1] = "0", "0total_voltage"
-        value = "VALUE = {V(" + str.format("{}out" , len(L)) + ")-V(1in)}"
+        value = "VALUE = {V(1in)-V(" + str.format("{}out" , len(L)) + ")}"
         netlist.add(GeneralElement(name, nodes, value));
 
         # Add a resistor between local and global ground for a bias point calculation
@@ -1785,6 +1785,8 @@ class FrequencyModelling:
         netlist.add(CommentElement(" Voltage source"));
         name = "V_AC"
         nodes[0], nodes[1] = "1out", "1in"
+        #nodes[0], nodes[1] = "1in", "1out" # matched based on the D1 group model
+        #nodes[0], nodes[1] = nodesL[0][0], nodesR[len(L) - 1][1]
         value = "AC 1"
         netlist.add(GeneralElement(name, nodes, value));
 
@@ -1792,7 +1794,8 @@ class FrequencyModelling:
         netlist.add(CommentElement(" Voltage measurement"));
         name = "E_total_voltage"
         nodes[0], nodes[1] = "0", "0total_voltage"
-        value = "VALUE = {V(1out)-V(1in)}"
+        value = "VALUE = {V(1in)-V(1out)}"
+        #value = "VALUE = {V(" + str.format("{}out" , len(L)) + ")-V(1in)}"
         netlist.add(GeneralElement(name, nodes, value));
 
         # Add a resistor to ground for a bias point calculation
@@ -1848,6 +1851,8 @@ class FrequencyModelling:
 
             Lib = [sub.replace('.PARAM', '+ PARAMS:') for sub in Lib]
 
+            #replace capacitance nodes
+            Lib = [sub.replace('C_total (1in 1out)', 'C_total (00_IN 00_mGND)') for sub in Lib]
             #Lib = [sub.replace('(0', '(00_mGND') for sub in Lib]
             #Lib = [sub.replace(' 0)', ' 00_mGND)') for sub in Lib]
             Lib = [sub.replace('1in', '00_IN') for sub in Lib]
